(**************************************************************************)
(*                                                                        *)
(*  Support modules for network applications                              *)
(*  Copyright (C) 2014   Peter Moylan                                     *)
(*                                                                        *)
(*  This program is free software: you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License as published by  *)
(*  the Free Software Foundation, either version 3 of the License, or     *)
(*  (at your option) any later version.                                   *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  along with this program.  If not, see <http://www.gnu.org/licenses/>. *)
(*                                                                        *)
(*  To contact author:   http://www.pmoylan.org   peter@pmoylan.org       *)
(*                                                                        *)
(**************************************************************************)

IMPLEMENTATION MODULE Debug;

        (************************************************************)
        (*                                                          *)
        (*            Looking after text-based INI data             *)
        (*                                                          *)
        (*    Started:        20 June 2009                          *)
        (*    Last edited:    20 June 2009                          *)
        (*    Status:         Now working, I believe                *)
        (*                                                          *)
        (************************************************************)

IMPORT FileSys;

FROM FileOps IMPORT
    (* type *)  ChanId,
    (* proc *)  OpenNewFile, CloseFile, FWriteChar, FWriteString,
                FWriteLn;

(************************************************************************)

CONST DebugFileName = "DEBUG.TXT";

VAR
    (* ChanId used for debug messages. *)

    debugcid: ChanId;

(************************************************************************)
(*                        MISCELLANEOUS UTILITIES                       *)
(************************************************************************)

(*
PROCEDURE WriteCard (N: CARDINAL);

    BEGIN
        IF N > 9 THEN
            WriteCard (N DIV 10);
            N := N MOD 10;
        END (*IF*);
        STextIO.WriteChar (CHR(N + ORD('0')));
    END WriteCard;

*)

(************************************************************************)
(*                          DEBUGGING STUFF                             *)
(************************************************************************)

PROCEDURE OpenDebugFile;

    VAR dummy: BOOLEAN;

    BEGIN
        IF FileSys.Exists (DebugFileName) THEN
            FileSys.Remove (DebugFileName, dummy);
        END (*IF*);
        debugcid := OpenNewFile (DebugFileName);
    END OpenDebugFile;

(************************************************************************)

PROCEDURE CloseDebugFile;

    BEGIN
        CloseFile (debugcid);
    END CloseDebugFile;

(************************************************************************)

PROCEDURE DebugCid(): ChanId;

    (* Returns the channel ID for the open debug file. *)

    BEGIN
        RETURN debugcid;
    END DebugCid;

(************************************************************************)

PROCEDURE DebugText (text: ARRAY OF CHAR);

    (* Write text. *)

    BEGIN
        FWriteString (debugcid, text);
    END DebugText;

(************************************************************************)

PROCEDURE DebugLine (text: ARRAY OF CHAR);

    (* Write text plus newline. *)

    BEGIN
        FWriteString (debugcid, text);
        FWriteLn (debugcid);
    END DebugLine;

(************************************************************************)
(*                             INITIALISATION                           *)
(************************************************************************)

BEGIN
    OpenDebugFile;
FINALLY
    CloseDebugFile;
END Debug.

