:userdoc.
:title.SFTPServer documentation
:docprof toc=123.

.***********************************
.*   INTRODUCTION
.***********************************

:h1 res=1000 id=1000 global.Introduction

:p.This package is an SFTP server, similar to an FTP server but using an encrypted
transmission channel. Its features are broadly based on the (non-encrypted) server
called FTPServer.

:p.It is now essentially complete. Feedback on your experience is welcome.
 If you wish to join a discussion list for FtpServer and SFtpServer, send
 an e-mail to majormajor@os2voice.org. The subject line is not important
 and may be anything. In the body of the message, put the lines

:xmp.       subscribe ftpserver-list
       end
:exmp.

:p.To have yourself removed from the list, send a similar e-mail but using the command "unsubscribe" instead of "subscribe".

:p.The author of this package is Peter Moylan, peter@pmoylan.org.

.***********************************
.*   CONFIGURATION
.***********************************

:h1.The Setup program
:hp2.The Setup program:ehp2.

:p.You configure the server by running Setup.exe. This is described in a
:link reftype=hd database='SETUP.INF' refid=1000.separate manual:elink..

:p.Setup is designed to configure FtpServer, or SFtpServer, or both. In the
'both' case, it assumes that identical settings (where possible) are wanted
for both servers. You may, however, designate individual users as FTP-only
or SFTP-only.

:p.Note that a number of FTP items are not applicable to SFTP. The two
protocols are rather different from each other, so they don't use all
of the same options.

:p.The first time you run Setup, you should use the command
:xmp.    setup -D
:exmp.

:p.This lets you specify the directories for the two servers, including whether
either or both are running on a different machine, and the language used
for the notebook labels. Once you have set these up, you don't need the -D
option unless you need to change those options.

.***********************************
.*   PROJECT STATUS
.***********************************

:h1 id=projectstatus.Overall project status
:hp2.Overall project status:ehp2.

:p.The encryption substrate is an implementation of the secure shell SSH. This is working,
and described in a
:link reftype=hd database='SSH.INF' refid=1000.separate manual:elink..

:p.Enough of the SFTP protocol is implemented to do all of the things
a client would want to do, so this is a production version. The only features
not implemented are those to do with setting file attributes and those
related to symbolic links. In the FTPServer virtual filesystem model,
permissions and links are under control of the system manager, so allowing
users to change these would be considered to be a security violation.

:p.:hp2.SFTP commands that are working:ehp2.
:ul compact.
    :li. SSH_FXP_INIT       (initialisation)
    :li. SSH_FXP_VERSION    (report SFTP version)
    :li. SSH_FXP_OPEN       (open a file for reading or writing)
    :li. SSH_FXP_CLOSE      (close a file or directory handle)
    :li. SSH_FXP_READ       (download)
    :li. SSH_FXP_WRITE      (upload)
    :li. SSH_FXP_LSTAT      (report file attributes)
    :li. SSH_FXP_FSTAT      (attributes of an open file)
    :li. SSH_FXP_OPENDIR    (prepare a directory for reading)
    :li. SSH_FXP_READDIR    (directory listing)
    :li. SSH_FXP_REMOVE     (delete a file)
    :li. SSH_FXP_MKDIR      (create a directory)
    :li. SSH_FXP_RMDIR      (remove a directory)
    :li. SSH_FXP_REALPATH   (canonicalise path names)
    :li. SSH_FXP_STAT       (report file attributes)
    :li. SSH_FXP_RENAME     (rename a file or directory)
:eul.

:p.:hp2.SFTP commands that will probably never be implemented:ehp2.

:ul compact.
    :li. SSH_FXP_SETSTAT    (set attributes of a closed file}
    :li. SSH_FXP_FSETSTAT   (set attributes of an open file)
:eul.
:p.These are to alter things like read/write/delete permissions. In the
FTPServer file system model, those permissions can only be changed by the
system manager; we do not want SFTP clients to be able to modify them.
Perhaps a subset could be implemented, for example to allow a file to
be truncated by changing its size.

:ul compact.
    :li. SSH_FXP_READLINK   (reveal the target of a symbolic link)
    :li. SSH_FXP_SYMLINK    (create a symbolic link)
:eul.
:p.These are again inconsistent with the FTPServer file system model,
where links are hidden from FTP clients because they are use to control
which directories are visible to clients.

.***********************************
.*   THE KEYGEN UTILITY
.***********************************

:h1 id=keygenutility.The keygen utility
:hp2.The keygen utility:ehp2.

:p.The server needs a public/private RSA key pair so that it can produce a
cryptographic signature during the initial negotiation with a client. The program
keygen.exe creates this key pair, so you need to run it before your first run
of the server. It is invoked with
:xmp.
       keygen -b nnnn
:exmp.
where nnnn is the desired number of bits in the public key. If it is not specified
then the default value is 1536. The reason for this default is that many people
now think that 1024 bits or fewer is not enough for security. You are of course free
to experiment. The result is stored in a subdirectory with name .ssh.

:p.The starting point for generating the key pair is finding two large random prime
numbers. This is a very slow operation because most of the candidates turn out not to
be prime, and discovering that is time-consuming. (The tests I've done suggest that
the execution time for keygen is an exponential function of the number of bits.)
Luckily, this only needs to be done once.

:p.If you have RSA key files called id_rsa and id_rsa.pub generated by other software,
you might be able to copy them into the .ssh subdirectory of the server, but
this is not guaranteed to work, because not everyone uses the same format
for storing RSA keys.

:p.:hp2.Other utilities:ehp2.

:p.In the zip file you will find these other utilities.
:ul.
:li.dumpkey and dumppublickey, for writing the generated key to standard output,
in case you're not sure whether keygen worked as desired.
:li.keygentime, a timing test. It produces a comma-separated list suitable for
importing into a spreadsheet, so that a graph can be produced if desired.
NOTE: takes a long time, so best run overnight. My tests so far suggest that
the time is an exponential function of the number of bits in the public key.
:eul.

.***********************************
.*   RUNNING THE SERVER
.***********************************

:h1 id=runningserver.Running the server
:hp2.Prerequisites:ehp2.

:p.The program assumes that INIData.DLL and XDS230M.DLL are somewhere on
your LIBPATH. If you get a message about a missing DLL, download the
INIData package (freeware), available in the same places where you
found this program.

:p.The server requires you to have an RSA public/private key pair, by which
it identifies itself to SFTP clients. You must therefore run KEYGEN.EXE
before the first time you run SFTPD.EXE. From then on you can keep the same
key. You might need to regenerate a key if clients complain that your key
doesn't have enough bits.

:p.:hp2.Running the server:ehp2.

:p.The server is run with the command
:xmp.     sftpd
:exmp.

:p.There are no parameters.

:p.It is safe to run FTPServer and SFTPServer at the same time, because they
use different ports. However, you should probably not put them in the same directory.

:euserdoc.

