:userdoc.
:title.Documentation for SFTPServer/FTPServer Setup
:docprof toc=123.

.***********************************
.*   INTRODUCTION
.***********************************

:h1 res=1000 id=1000 global.Introduction
:p.
FTPServer and SFTPServer are FTP daemons for OS/2. The difference
between them is that SFTPServer uses an encrypted channel for all operations, including data transfers,
and the two servers are based on quite different standards.
What they have in common is a user permissions model that allows
each client to have a different view of the file system, with the
client not allowed to move outside the directories specified by the
system manager.

:p.This documentation is for the Setup program for SFTPServer.
Optionally, this program can also be used to configure FTPServer.

:p.
:hp2.Disclaimer of Warranty:ehp2.

:sl compact.
:li.
:hp8.
This Product is provided "as-is", without warranty of any
kind, either expressed or implied, including, but not limited to,
the implied warranties of merchantability and fitness for a
particular purpose. The entire risk as to the quality and
performance of the Product is with you. Should the Product prove
defective, the full cost of repair, servicing, or correction lies
with you.
:ehp8.
:esl.

:p.
The author of this software is Peter Moylan, peter@pmoylan.org.

:p.
The latest version of SFTPServer is normally kept at ftp&colon.&slash.&slash.ftp.pmoylan.org/SFTPServer
or sftp&colon.&slash.&slash.ftp.pmoylan.org/SFTPServer
or http&colon.&slash.&slash.www.pmoylan.org/ftp/SFTPServer&slash.
.br
(These multiple access methods had to be provided after web browser developers
rather stupidly broke their support for the ftp&colon.&slash.&slash. form of URL.)
Information about other software on this site may be found at
http&colon.&slash.&slash.www.pmoylan.org/pages/os2/software.html.

:p.
You may, if you wish, join a mailing list for discussions and questions
about FTPServer and SFTPServer. To join the list, send an
e-mail to majormajor@os2voice.org. The subject line is not
important and may be anything. In the body of the message, put the
lines
:xmp.       subscribe ftpserver-list
       end
:exmp.

:p.To have yourself removed from the list, send a similar e-mail but
using the command "unsubscribe" instead of "subscribe".

.***********************************
.*   PREREQUISITES AND INSTALLATION
.***********************************

:h1 id=prerequisites.Prerequisites and installation

:hp2.Prerequisites:ehp2.

:p.This software assumes that both INIDATA.DLL and XDS230M.DLL are in your
LIBPATH. If, when trying to run Setup.exe, you get a message like
"The system cannot find the file XDS230M", you must install INIData,
version 1.0 or later. (With older versions of OS/2, it needs to be version 1.1
or later.) INIData can be found at the same web or FTP site as where
you found the SFTPServer zip file.

:p.:hp2.Installation:ehp2.

:p.Installation of Setup is just a matter of unzipping the Setup program.
You may, if you wish, unzip it into the same directory as SFTPServer.
Alternatively, you may give it its own directory. Wherever you put it, the
setup.*.lng files must go into the same directory.

.***********************************
.*   RUNNING THE SETUP PROGRAM
.***********************************

:h1 id=runsetup.Running the Setup program

:hp2.Running Setup for the first time:ehp2.

:p.Initially, run the program with the command
:xmp.      SETUP -D
:exmp.
This allows you to specify the directories where FTPServer and SFTPServer live.
(They should not be in the same directory.) You also specify whether you will
be configuring FTPServer, or SFTPServer, or both; and the language for the
labels in the Setup notebook.

:p.:hp2.Running Setup normally:ehp2.

:p.After that, you can run Setup without any parameters. Of course, you may use
the -D parameter any time you want to go back and change those initial settings.

:p.:hp2.The INI vs TNI choice:ehp2.

:p.SETUP.EXE is, in effect, an editor for the server's configuration options.
Those options are stored in either TNI format (a plain text file) or INI format
(a binary file). If you want to override the default choice, you can state Setup
with a -T parameter (to specify TNI format) or an -I parameter (to specify INI
format). Normally you don't need those parameters, because Setup will, in effect,
choose whatever it was using last time.

:p.More precisely, the choice between whether to use FTPD.INI or FTPD.TNI,
when there is no explicit -T or -I parameter, is made as follows:

:ul.
:li.If only one of those files exist, then that will be the one used.
:li.If both exist, then a flag inside those files is used to specify
which one should be used.
:li.If this still does not produce a decision, an arbitrary decision is made.
:eul.

:p.If you are using Setup to configure both servers, then that decision is
made independently for the two servers, provided that they are installed in
separate directories.

.***********************************
.*   THE TWO SERVERS
.***********************************

:h1 res=002.The two servers

:p.FTPServer and SFTPServer are both file transfer programs. FTPServer is faster,
because it transfers everything in unencrypted form. SFTPServer uses encryption,
so is more secure. Which you should use depends on what level of security you want.
You can run both servers in parallel. An obvious arrangement would be to keep your
more confidential files in directories that only SFTPServer can see.

:p.:hp2.FTPServer:ehp2.

:p.FTPServer is based on the standard RFC 959, and later updates to that standard.
It is what you need to do FTP with traditional FTP clients.

:p.:hp2.SFTPServer:ehp2.

:p.SFTPServer is based on version 3 of the SFTP draft standard, as described in
a document you can find by searching the web for draft-ietf-secsh-filexfer-02.
It is radically different from the original FTP standard, but supports approximately
the same operations. It gets its security by being implemented on top of SSH,
a "secure shell" described in RFC 4251 to 4254. The SSH implementation is
built into the SFTPServer implementation, so you do not have to install a
separate SSH program. Note, however, that this implementation supports ONLY SFTP.
Other SSH features such as a remote terminal are not included.

:p.Although SFTPServer implements most of the draft standard, it omits two features:
:ul.
:li.It does not allow SFTP clients to change file permissions.
:li.It does not allow symbolic links to be manipulated.
:eul.

:p.Those features are left out because they are inconsistent with the FTPServer
file model, and are (in my opinion) serious security holes.

.***********************************
.*   DEINSTALLATION
.***********************************

:h1 id=deinstall.De-installation
:hp2.De-installation:ehp2.
:p.
Setup does not tamper with CONFIG.SYS or with other system files.
It modifies only three files:
:ul.
:li.SETUP.INI and/or SETUP.TNI, in its own directory.
:li.FTPD.INI and/or FTPD.TNI, in the FTPServer directory.
:li.FTPD.INI and/or FTPD.TNI, in the SFTPServer directory.
:eul.
:p.If you decide that you don't want to keep this program, simply delete
the unwanted files.

.***********************************
.*   SETTING UP THE USER PERMISSIONS
.***********************************

:h1 id=setupusers.The FTPServer file model

:hp2.User Permissions:ehp2.

:p.Unlike conventional FTP servers, FTPServer and SFTPServer do not allow
FTP clients to see the layout of the host file system. Instead, a client
sees a virtual file system, starting with a root '/', that contains
only those directories that the system manager allows that client to see.
This can be different for different clients.

:p.If needed, the manager can define symbolic links that cross to different
parts of the file system. That detail is, however, invisible to the user. The user sees
only a single directory tree.

:p.That view is, of course, defined by the Setup program.

:ul compact.
:li.:link reftype=hd refid=UPconcepts.General concepts:elink.
:li.:link reftype=hd refid=PMconfiguser.Setting up users with the Setup program:elink.
:li.:link reftype=hd refid=manager.Manager privileges:elink.
:li.:link reftype=hd refid=UsingTemplates.Using templates:elink.
:eul.

.***********************************
.*   GENERAL CONCEPTS
.***********************************

:h2 id=UPconcepts.User permissions: General concepts

:hp2.GENERAL CONCEPTS:ehp2.

:p.
Each user of the server has a login name (username), a password, and
a tree of accessible directories. Typically this tree consists of
one home directory and all of its subdirectories, but more complex
arrangements are possible. The system manager may, by the use
of symbolic links, allow the tree to cross multiple drives or even
multiple nodes on the local area network. The system manager may also
allow access to a directory but block access to specified subdirectories
of that directory. In all cases, a user is restricted to using the
directories that the system manager has specified for that user. Users
cannot get at, or even see, any other directories in the machine's file
system. Furthermore, users are not told the true physical paths of
those directories that they are allowed to see.

:note.
In this context, "user" refers to a username rather than to
a person. For example, you might have a number of different
people all accessing the server via the username "anonymous".
As far as the server is concerned, they are not separate users,
but rather separate instances of the user called "anonymous".

:p.
:hp2.User categories:ehp2.

:p.
Each user is classified as one of the following.

:dl break=fit.
:dt.   no password
:dd.User who does not need a password.
:dt.   guest
:dd.Guest user, who has to provide an e-mail address as a password
in the FTP case. The SFTP server does not enforce this, and does
not ask for a password.
:dt.   user
:dd.Normal user, who has to supply a password
:dt.   manager
:dd.Manager: same as a user, except that a manager gets some
:link reftype=hd refid=manager.extra privileges:elink..
:dt.   template
:dd.Template. Unlike the categories above, this is not a
client account that users can log in to. Instead, it is a
way of defining a set of directory permissions that can be shared by
a group of users. See also the section
:link reftype=hd refid=UsingTemplates.Using templates:elink..
:edl.

:p.
Normally you would create one "manager" account for yourself, and use
the guest or user categories for all other users. The nopassword category is for those
rare cases where you don't need to control access with a password.

:p.
:hp2.Directory permissions:ehp2.

:p.
Each directory that is accessible to the user is described by some
combination of the following five permission attributes.

:dl break=none.
:dt.   V
:dd.Directory visible. This should be set in most cases. When it is not
set, the client can't do a "change directory" to this directory, and it
will not appear in directory listings.
:nt text='Special case'. It is always legal to ask for a listing of the root
directory, but if it is invisible then the result will be an empty listing.
It is also legal to change directory to the root directory if it is
invisible, even though it is not possible to change to any other
invisible directory.
:ent.
:dt.   R
:dd.Read permission. If this is set then the client can download files
from this directory.
:dt.   W
:dd.Write permission. If this is set then the client can upload files
to this directory.
:dt.   D
:dd.Delete permission. If this is set then the client can delete files
from this directory. Note: to overwrite an existing file, both W and D
permissions are needed.
:dt.   N
:dd.Rename permission. If this is set then the client can rename files
in this directory. Note: if the rename results in moving a file to a
different directory, then the N permission flag is no longer relevant. In that
case, the user needs a D permission for the source directory and a
W permission for the destination directory.
:edl.

:nt text='Remark'. It is possible for a user to be given read, write,
delete, and/or rename privileges to an invisible directory. In such cases the users can
perform the permitted operations only if they know the correct file name,
including the directory name,
because they won't see the directory name in a directory listing.
:ent.
:p.
It is also possible for an invisible directory to have visible subdirectories.
Users can get to those directories only if they know the path name, including
the name of the invisible directory.

:p.Another way of hiding files and directories, independently of the
V attribute, is to use a HideList. This is explained in the section on
:link reftype=hd refid=PMedituser.Editing a user's permissions:elink.

:p.
:hp2.Symbolic links:ehp2.
:p.
A symbolic link is a pointer to some other part of the machine's file system.
You - the system manager - can insert symbolic links in any directory.
From the user's point of view, a symbolic link looks like just another
file or subdirectory.
:p.
A symbolic link has a name (which is what the user sees) and a physical
path (which is known only to the system manager). The physical path must
be either a null string, or a full path name including the drive letter.
:p.
A symbolic link normally points to a directory, but it is also possible
to create a symbolic link to a file that is not a directory.
:p.
Note that the user cannot see any difference between a symbolic link and
an ordinary file or subdirectory name. All that the user sees is a single
directory tree that starts with a root node called "/".

:p.
:hp2.Pseudo-directories:ehp2.
:p.
It is possible to specify a symbolic link whose physical path is unspecified.
(That is, it is an empty string.) This creates a pseudo-directory:
something that ftp clients will see as a directory, but which does not
correspond to any physical directory. A pseudo-directory cannot hold any
real files, but it can contain symbolic links.
:p.
The main use for a pseudo-directory is for the case where you want to give
a user access to several unrelated directories, possibly on different
drives, or even on network drives. To do this, you can make the user's top-level directory a
pseudo-directory, and then put links in that directory to the directories
that the user is allowed to see. More complicated examples, using
pseudo-directories deeper down in the directory tree, are of course possible.

.***********************************
.*   MANAGER PRIVILEGES
.***********************************

:h2 id=manager.Manager privileges
:hp2.Manager privileges:ehp2.
:p.
A manager account is the same as a normal user account, except that
a manager has a few extra privileges.
:ul.
:li.Managers can see system and hidden files in directory listings;
other users cannot.
:li.For FTPServer only, managers are allowed to use the SITE MNGR commands.
The SFTP protocol does not allow anything equivalent to SITE MNGR.
:eul.

.***********************************
.*   USING TEMPLATES
.***********************************

:h2 id=UsingTemplates.Using Templates
:hp2.Using Templates:ehp2.
:p.
A template is a way to allow groups of users to share the same
directory permissions. In the Setup program, you can create a
pseudo-user of type "Template". This is not a true user account,
because nobody may use the name of this template as a login name.
You can think of it as either a placeholder or as a way to define
groups. You create the directory permissions for a template in
exactly the same way as you would create them for a normal user
account.

:p.The non-directory attributes of a template account (password,
user limit, speed limit, etc.) are ignored by the software, so the
values that you give those attributes are of no importance.

:p.To use a template, you can go to any other account in the
Setup program and check the "Use Template" box. You must then
enter the name of the template. Once you do this, you will find
that you can no longer edit the directory permissions for that
user. Instead, the directory permissions are a direct copy of
those in the template. If you change the permissions in the
template, they will automatically change for all user accounts
that are using that template.

.***********************************
.*   THE SETUP INITIAL SCREEN
.***********************************

:h1 id=initialscreen.The Setup initial screen
:hp2.The Setup initial screen:ehp2.

:p.When you start Setup with the -D parameter, you get a window where
you specify, via the "Enable" options, whether you are configuring
FTP or SFTP, or both. In each case, you have to specify the directory
in which that server lives. It would be unwise to put them both in
the same directory.

:p.The directories have to be specified as full path strings,
including a drive letter.

:p.You may also specify a language code, for example "nl" for Dutch.
This controls the labels in the Setup notebook. A language xyz is supported
provided that there is a file setup.xyz.lng in the Setup directory. A
new language can be added by copying one of the existing setup.*.lng files,
and editing the resulting file in a plain text editor. If you do this,
I would be happy to include your translation (with appropriate credit)
in future releases.

.*************************************************************************
.*   REMOTE CONFIGURATION
.***********************************

:h1 id=remoteconfig.Remote configuration

:hp2.Remote configuration:ehp2.

:p.The initial screen offers a "Remote" option for each server. That
is, you can run Setup on one computer and use it to configure a copy
of FTPServer or SFTPServer that is installed on a different computer. To do this, you have
to have the freeware utility INIServe running on the same computer as
the server. You can find INIServe at http&colon.&slash.&slash.www.pmoylan.org&slash.pages&slash.os2.

:p.If you enable the "Remote" option, you will get a new window with fields

:dl break=all.
:dt.    Hostname
:dd.The name (or IP address) of the machine on which FTPServer or SFTPServer, is running.
(Setup does not currently allow for two servers running on two different machines.)
:dt.    INIServe port
:dd.The TCP port that INIServe has been configured to listen on. The default
value is 3560.
:dt.    INIServe password
:dd.The password needed to log in to your copy of INIServe.
:edl.

:p.When you close this subwindow, those settings are stored. If you later want
to change them, just disable the Remote option and then re-enable it.

:p.IniServe on the remote machine is contacted when you click on the OK
button. If this gives a "failed to connect" or similar error
message, it probably means that you don't have INIServe running on the
remote machine, or that you have done something like specifying an incorrect
port number or incorrect password.

:p.Once the connection is made, the operation is the same as for the
case of local configuration.

:p.When doing remote configuration, the decision as to whether to use Setup.INI or Setup.TNI
is based on local files, because you are still using a local copy of Setup. On
the other hand, the choice between FTPD.INI and FTPD.TNI is based on those files,
if they exist, at the remote location. This is because we want to make the same choice as
will be made by the remote FTPD.EXE or SFTPD.EXE.

.***********************************
.*   THE SETUP MAIN NOTEBOOK
.***********************************

:h1 id=PMconfiguser.The Setup main notebook
:hp2.The Setup main notebook:ehp2.

:p.When you run Setup, you get a notebook that controls all of the
configuration details of the server. The parameter settings are stored
in a file FTPD.INI or FTPD.TNI. When you change these parameters, existing user
sessions will continue, in most cases, to use the old settings, but
future clients will be affected by the changes. This means that you
can do things like adding users or changing user permissions while
the server is running.

:p.There are certain parameters that cannot be changed while the
server is running. These are:
:ul.
:li.the server port number
:li.the user log options (FTP only)
:li.the option to bind the server to a specific IP address
:li.the free space threshold (FTP only), which determines whether there is
enough free disk space to permit an upload to be performed
:eul.

:p.You may still change these parameters in the Setup program, but
if the server is running at the time then
the changes will not have any effect until after you have shut down
and restarted the server.

:p.Note that there are many options that apply only to FTP, not to SFTP.
This is because the SFTP standard is so different from the FTP standard
that those options are unavailable or irrelevant.

:p.The setup details are divided into several groups.

:ul compact.
:li.:link reftype=hd refid=PMconfiguser1.Basic:elink.
:li.:link reftype=hd refid=PMlogging.Logging:elink.
:li.:link reftype=hd refid=PMconfiguser1a.Options:elink.
:li.:link reftype=hd refid=LocalAddrs.Local addresses:elink.
:li.:link reftype=hd refid=PMsecurity.Security:elink.
:li.:link reftype=hd refid=PMconfiguser2.Users:elink.
:eul.

.***********************************
.*   THE BASIC SETUP SETTINGS
.***********************************

:h2 id=PMconfiguser1.The basic server settings
:hp2.The basic server settings:ehp2.
:p.
The "Basic" page controls the following parameters.

:dl break=all.
:dt.FTP Port
:dd.This is the tcp port on which FtpServer listens for new connections.
Unless you are doing something nonstandard (for example, running two ftp servers
on the same machine) this should always be 21.
:dt.SFTP Port
:dd.This is the tcp port on which SFtpServer listens for new connections.
It is normally 22.
:dt.Maximum number of users
:dd.This specifies how many clients will be allowed to use each server simultaneously.
I usually set this to 10, but you would set the limit much higher if you are
running a dedicated server for many clients. Higher values will, of course, increase the potential
load on your processor.
:p.Note: this is a global maximum. You may also set this to a high value, and then
control the number of users on a per-username basis.
:dt.Maximum number of guest users (FTP only)
:dd.This typically should be less than the number specified for the maximum
number of users, to reserve one or more login slots for the system manager and other
non-guest users.
:dt.Free space threshold (MB) (FTP only)
:dd.This specifies the amount of free space that must be available on a drive for
uploads to be enabled. If the free space, in megabytes, falls below this level then
uploads will be disabled.
:dt.Timeout (seconds)
:dd.The time that a client session may remain idle before the user is evicted.
You will find that many ftp and sftp clients, especially web browsers, don't log out properly,
so their sessions have to be killed with the timeout mechanism. You may specify an
interval of up to 4294967 seconds (about 49 days), but normally you would want to
eject idle clients much earlier than that. In most situations a value of 300
seconds (5 minutes) is a suitable compromise. Do not set the value too much lower
than that, because you have to allow for human response time.
:p.Note that the time spent uploading or downloading a file is not counted as
idle time. Even if that transfer takes many hours, there will not be a timeout
provided that each "chunk" of that file is transferred reasonably often.
:edl.

:p.Below this the two server directories are shown. This is for information only.
If you want to change these, you have to run SETUP -D.

:p.Next, we have a couple of checkboxes.
:dl break=all.
:dt.Use multiple INI files
:dd.Normally the data for all user accounts are kept in the main INI file called
FTPD.INI or FTPD.TNI, and most people will want to leave it that way. With this option activated, the account
data are spread over many different INI files, which can improve
access speed in the case when you have hundreds or
thousands of ftp user accounts.

:note.Here and elsewhere in this manual, any reference to "INI files" refers
to files that can be either in INI or TNI format.

:p.Note that switching this option on or off forces a major rearrangement
of the INI data. This can take a long time, especially when working on a
large INI file. You should wait for the conversion to finish before
exiting from the Setup notebook.
:dt.Disable Telnet compatibility (FTP only)
:dd.This option is for use with servers using the Windows-1251 character
set to support file names in Cyrillic. Everyone else should probably
leave it turned off. The FTP standard requires servers to be compatible
with the standard for Telnet; that is, there might be Telnet commands
embedded in the command stream. (In particular, FTP clients are allowed
to generate the Telnet "Interrupt Process" command.) The first byte of
a Telnet command is a byte with code 255 (0FF hexadecimal). This normally
causes no conflict, because that code is unused - because it is reserved for Telnet - in all standard character
sets. Unfortunately, it is the code for the Cyrillic letter 'ya' in
the nonstandard code Windows-1251. That means that Russians, and anyone
else using Cyrillic letters, must choose between doing without Telnet
and doing without the letter 'ya'. Most of them have chosen to do without
Telnet. If this applies to you, you must activate this option in order
to have access to files whose names include the letter 'ya'.

:p.In principle, a better solution is to abandon Windows-1251 and move
to a standard character code such as ISO-8859-5. In practice, Microsoft
has won its battle against standards in Eastern Europe, and the ISO codes
are not popular with users.
:edl.

.***********************************
.*   LOGGING
.***********************************

:h2 id=PMlogging.Logging
:hp2.Logging:ehp2.

:p.FtpServer can produce three different log files. You can enable any
combination of these three.
:ul.
:li.The user log file contains a summary of the upload and download
information for each user session.
:li.The "common log" contains roughly the same information as the user
log, but in a format that is commonly used by web servers. You can use
this format if you use web server analysis tools to produce a
periodical summary of the transfers.
:li.The transaction log is a more detailed log of every operation
performed by the clients. If you write the transaction log to disk,
you can use it for troubleshooting operations such as tracing
attempts to violate your system security. You should be aware,
however, that detailed logging can produce very large log files.
If you use this option, you should archive or delete the log files
at frequent intervals. If you write the log information to the
screen, you can get an idea of how busy your ftp server is.
:eul.

:p.Transfer logging is more difficult with the SFTP protocol, so
SFtpServer only has the transaction log option.

:p.You may safely delete or move the log files. The server
re-creates the files if they are missing. It would be a good idea
to move these files to an archive directory once a month, or more
frequently if you have a busy server. There are many available
software products (see, for example, DragText) that can schedule
jobs to be done once per month, or once per week, etc.

:p.:hp2.Transfer logging (FTP only):ehp2.
:p.You can ask the server to
produce a user log (FTPUSERS.LOG) and/or a log in common log format (COMMON.LOG).
(If you delete these files, they will be re-created. It would be a good idea to
delete them periodically, or move them to an archive, so that they do not grow
too large.) The user log produces a list of files that have been uploaded
or downloaded. The common log contains similar information, but in a format
used by many http servers. This allows you to use log analysis tools that
have been designed for web servers.

:p.You can select how much detail gets written to these two logs.

:dl break=all.
:dt.    No transfer logging
:dd.This effectively disables the transfer logging.
:dt.    Log successful transfers
:dd.The log includes entries for all uploads and downloads that completed
successfully, but the operations that failed are not logged.
:dt.    Log all file transfers
:dd.With this option, you get log entries even for transfers that were
aborted before they completed.
:dt.    Log all clients
:dd.This creates user log entries for all users, even those who didn't transfer any files.
:edl.

:p.The names FTPUSERS.LOG and COMMON.LOG mentioned above are the default
names of the log files. As you will see when running the Setup program,
you have the option of specifying different names. Similarly, you are
free to change the name of the disk file that holds the transaction log (see below).
All three of the file names that can be specified on this page may include
directory specifications, either relative to the current working directory
or as absolute path specifications. This permits you to keep the
log files in different directories if that is what you prefer.

:p.:hp2.Transaction logging:ehp2.
:p.The transaction log is a much more detailed log. You can choose to send it
to the screen, to a disk file, to a pipe, to the syslog daemon, or any combination of these.
The disk files, called FTPTRANS.LOG and SFTPTRANS.LOG unless you change their names in the
setup notebook, are updated approximately once every
minute if this feature is enabled. While the server is running, you
might notice that a file with a name like FTPTRANS.$$$ is created. This
is a temporary file containing the most recent transaction log entries.
We distinguish between the short-term log and the final log for two
reasons: to avoid overheads associated with writing to large
files, and to avoid sharing violations that might otherwise occur if
you tried to read the transaction log while the server has it open.

:p.Warning: Transaction logging can create very large log files. I suggest that
you don't enable transaction logging to a file unless you are trying to track
down a problem, or unless you regularly delete the transaction log.
Logging to the screen, on the other hand, will give you some idea of how busy
the server is. Logging to the screen does add some processor overhead,
but you are unlikely to notice this unless you have a very busy server.

:p.If you choose the "pipe" option, the log is sent to a named pipe
called \PIPE\FtpServerTranslog. For this option to be useful, there
needs to be another program running that reads the data from the pipe.
Thus, this is a suitable option if you plan to write extra software
that analyses or reorganises the transaction log data. For some example
code on how to use a pipe, look for "pipelog.zip" in the "Weasel Tools"
section of the site where you got this program.

:p.If the syslog option is enabled, you would normally want to leave
the syslog host name set at 127.0.0.1 (the local host). You may, however,
log to a different machine by specifying a hostname or a numeric IP
address in the "Syslog host" field.

:p.For the "syslog" option to be useful, the syslog daemon should be
running on the machine specified as the syslog host. This can either
be the IBM-supplied syslogd, or any replacement
that reads its data from UDP port 514.

:p.The checkbox underneath the FTP transaction log name controls whether the
SITE MNGR commands are included in the transaction log (FTPServer only).

.***********************************
.*   MORE SETUP OPTIONS (PM)
.***********************************

:h2 id=PMconfiguser1a.Options
:hp2.Options:ehp2.

:p.Most of these options are FTP options, and are not relevant to SFTP.
The address binding option, however, can be specified separately for
FTP and for SFTP.

:p.:hp2.IP Address Binding:ehp2.

:p.The first option on this page is a choice between
binding to all local interfaces, or binding to a specific IP address. If you
choose the "specific address", you should enter an IP address in the
standard format (four decimal numbers, separated by dots). You make this
decision separately for FTP and for SFTP.

:p.For most applications, the best choice is "all interfaces". With this
choice the server listens for ftp or sftp requests, as appropriate on all your network interfaces,
even if your machine has multiple IP addresses. The most common reason
for having multiple addresses is that your machine has both a LAN
address - i.e. it is part of your local network - and a global IP
address that makes it visible to the global internet. If this is your
situation, the "all interfaces" option makes your server visible both
on the Local Area Network and on the global internet.

:p.The "specific address" option is for the case where you have two or more
IP addresses, but you want the server to respond to only one of them. In
this case you could, if you wished, run several independent ftp and/or sftp servers on
the same machine, each responding to a different address.

:p.(Another way to run several servers is to make each one listen on
a different port. That is a less attractive option, however, because most
ftp clients expect to find the server on the standard port&colon. 21 for ftp and 22 for sftp.)

:p.If you do run multiple copies of ftpd.exe of sftpd.exe, run them from
different directories. (This does not require having separate physical
copies of the executable.) This is because ftpd.exe and sftpd.exe expect to find their INI file
in the current working directory, and for multiple copies you
would want to have a different INI file for each one.

:p.:hp2.The 'Hide passwords' option:ehp2.

:p.If you check the 'Hide passwords' box, user passwords will not be
visible in the 'edit user' dialogue. In addition, password encryption will be
turned on by default when you add a new user (although you can choose
to turn it off again before entering the password).

:p.This is a reversible change. If you need to check someone's password,
you can turn this option off and then go to the 'Users' page. Note,
however, that there is no way to re-read a password that has already been encrypted.
If you unhide an encrypted password, all you will see is the encrypted form.
Hiding and encryption are two separate options.

:p.The remaining options on this page apply to FTP only.

:p.:hp2.Restricting the PASV port numbers (FTP only):ehp2.

:p.If you are running a firewall, or sometimes even if you do not have
a firewall, you might want to control which ports are used for a
passive data transfer. If you disable this option, you are trusting the
tcp/ip stack to choose the 'next available port' for a passive data
transfer. If you enable it, you restrict the possible port numbers that
can be chosen.

:p.Unless you have expert knowledge for setting up non-typical scenarios,
you should stick to the following guidelines:
:ol.
:li.For the majority of applications, the acceptable set of port numbers
lies in the range 49152 to 65535. (If you are running a firewall, and
you want greater security, you will probably want to use a smaller range. In that
case the range must agree with what the firewall will let through.)
Port numbers in the range 0 to 1023 should probably be never used, because
those ports are reserved for 'well-known services' such as SMTP and Telnet.
Port numbers from 1024 to 49151, inclusive, are used for servers that
are less 'official', but which nevertheless have well-defined ports. The
ports from  49142 onwards are available for non-server applications,
therefore these are the ones you should use for your data transfers.

:li.If you enable this option, the data port used by the server for
passive ftp is restricted to the specified range. This range should
match a range of ports that the firewall will let
through, and normally it should be a range that is not in use for some
other purpose. In addition, you must tell your firewall to forward
connections on these ports :hp3.without any port alteration:ehp3..
Note that this has no effect on non-passive ftp,
where port 20 is always used as the server's data port.

:eol.

:p.:hp2.Working behind a firewall (FTP only):ehp2.

:p.The 'behind firewall' mode of FtpServer is designed specifically to
deal with the case where you have a firewall or router that implements Network
Address Translation (NAT).

:p.Remark: in this section the term "firewall" is used, for brevity, to
mean any implementation of NAT. This could be a firewall, or a router, or
some combination of these. That is, the "behind firewall" option could be
needed even if you don't have a firewall.

:p.Note, especially, that some firewalls are smart enough to intercept
FTP commands and responses and to adjust them to compensate for the
address translation. If you have a firewall that implements NAT
completely, you should leave this FtpServer option turned :hp3.off :ehp3.. The
option is needed only if you have to compensate for the limitations of
your firewall. You should therefore enable this option only if you
discover that the server will not work with it turned off.

:p.If you need this option, you will probably also need to activate the
option to restrict the port numbers, as explained earlier on this
page.

:p.If the 'behind firewall' option is enabled, you will get the following
change:

:ul.

:li.A new
:link reftype=hd refid=LocalAddrs.Local addresses:elink.
page is added to the Setup notebook, to specify the details of the
"behind firewall" operation.


:li.In response to a PASV command, the server does not report its
own IP address. Instead, it reports the IP address of the
firewall. Since the server has no way of discovering the IP
address of the firewall, this must be entered manually in this
Setup program.

:eul.

:p.Note that this option affects only passive ftp. When we have
our server behind a firewall or router, we don't have to do anything special
to make non-passive ftp work correctly.

:p.:hp2.The anti-tagger option (FTP only):ehp2.

:p.This option was added in response to the actions of software
pirates, who appear to have ways of searching the world for FTP
servers that have write-enabled 'incoming' directories. They
upload pirated files, and then (presumably) tell other people
where to download those files from.

:p.If you enable this option, FtpServer checks for uploads that
are typical of pirate operations. It considers that a pirate has
been detected on any of the following conditions.
:ul.
:li.the creation of a directory whose name includes
the string 'tag'. This includes the name 'tag', but also
things like 'taGGed by kewl d00dz'
:li.the creation of a directory whose name is 12 digits followed by
the letter 'p'
:li.the creation of a directory without uploading any files into
that directory
:li.the uploading of a file whose name (modulo alphabetic case) is
SPACE.ASP or is a name including the substring TEST.PTF
:li.the uploading of a file whose name contains only numeric digits
or the '.' character
:eul.

:p.If any of these conditions is satisfied, FtpServer takes two actions&colon.
:ul.
:li.It immediately reduces the speed limit for this session to
a low value - not so low that the user will think the server has
hung, but low enough to be an inconvenience to the user.
:li.It allows the uploads to continue, but at the end of the session it
deletes all files and directories added during that session.
:eul.

:p.If you use this option, you should of course tell your
legitimate uploaders not to create subdirectories or upload files
whose names match the names used by the pirates.

.***********************************
.*   LOCAL ADDRESSES
.***********************************

:h2 id=LocalAddrs.Local Addresses
:hp2.Local Addresses (FTP only):ehp2.

:p.This page appears in the Setup notebook only if you have enabled
the "behind firewall" option.

:p.In this situation, the server's external address - that is, the address
seen by the client - is different from its internal address in the LAN.
The external address is known to the firewall or router, but FtpServer has
no way of finding it out. Therefore, you need to fill in the external address
in the first field on this page.

:p.The listbox below this specifies the client addresses that should
:hp3.not:ehp3. be subject to the "behind firewall" rules, usually because
they connect to the server via the LAN, without going through the firewall
or router. Most typically that listbox will require just two entries,
saying that you should allow a range of local addresses, and exclude all
others from the "local addresses" set. More complicated rules might be
needed if you have a complicated LAN.

:p.This list is edited in exactly the same way as the similar list on the
:link reftype=hd refid=PMsecurity.Security:elink. page.

.***********************************
.*   SECURITY SETTINGS (PM)
.***********************************

:h2 id=PMsecurity.Security settings

:hp2.Security settings:ehp2.

:p.There are separate security settings for FTPServer and SFTPServer.
If you are configuring both servers, you will see two security pages. These
are identical in their appearance and operation.

:p.The settings on this page allow you to put restrictions on the
addresses from which a client may connect to the server. You will want
to enforce such restrictions if, for example, you want the server
to be available only from inside your own LAN. Another common reason
for enforcing such restrictions is to lock out nuisance users who
are trying to use the server for unethical purposes.

:p.When a client is rejected as the result of the checks on this page,
there is a delay before the rejection message is sent back.
The reason for this delay is to control denial-of-service attacks,
where a hostile client keeps re-attempting to connect. The size of the
delay can vary from one version of the server to another.

:p.:hp2.Maximum simultaneous connections from a single client address:ehp2.

:p.
The first item on the "Security" page is a field called
"Max connections from same address".  This specifies the
maximum number of users that can be connected simultaneously from the same IP
address. It is primarily a protection against users who hog the server by
logging in more than once. I usually set it to 2 or 3. If you do not
want this protection, set the number to a very large value.

:p.:hp2.Restricting access to certain IP addresses:ehp2.
:p.
The large box on this page defines a filter for IP addresses. This is for
putting restrictions on which remote hosts are allowed to log into the server.
(If you don't need this feature, just use a single "Allow all" entry.) When a
client tries to connect, the server searches this list, starting at the
beginning, for the first entry that matches the client's IP address. There will
always be a match, because the last entry is always an "everything else" entry.
The allow/refuse flag on the matching entry is used to decide whether the
client should be allowed to connect. If the flag value is "refuse", the
connection attempt is rejected.

:p.The Edit, Insert, Promote, and Delete buttons have the obvious meanings.
The details of how an individual entry is specified are given on the
:link reftype=hd refid=IPFilter.following page:elink..

:p.The Copy and Paste buttons allow you to copy one or more entries to
a different place in the same list, or to copy between the two Security pages.
On a Paste operation, the copied entries are pasted as a block just
before the first selected list item. If no list item is selected, the
paste is done at the beginning of the list.

:p.Because the list is always searched from the beginning, and because
the search stops as soon as a match is found, the order of
entries in the list is important. In most cases you should put the
most specific rules at or near the beginning of the list, and the more
general rules later in the list.

:p.:hp2.Example 1.:ehp2. If you want to lock out all machines with IP addresses in
the range 123.45.67.0 to 123.45.67.127, your list would look like this.
:sl compact.
:li.Refuse   123.45.67.0   255.255.255.128
:li.Allow    all others
:esl.
:p.Equivalently, you could write this as
:sl compact.
:li.Refuse   123.45.67.0 / 25
:li.Allow    all others
:esl.

:p.:hp2.Example 2.:ehp2. Suppose you want to give access only to your local network,
which has addresses in the range 123.45.66.0 to 123.45.67.255. You can do this as
follows.
:sl compact.
:li.Allow    123.45.66.0 / 23
:li.Refuse   all others
:esl.

:p.:hp2.Example 3.:ehp2. To allow access to 123.45.67.89, but to lock out everyone
else in 123.45.67.*, you can use the rules
:sl compact.
:li.Allow    123.45.67.89
:li.Refuse   123.45.67.0    255.255.255.0
:li.Allow    all others
:esl.

:p.Notice that the list always finishes with an "all others" entry. The Setup
program will allow you to change the allow/refuse flag on this final entry, but
it will not allow you to delete it.

.***********************************
.*   IP FILTER ELEMENTS
.***********************************

:h3 id=IPFilter.Form of an address filter list element

:hp2.Form of an address filter list element:ehp2.

:p.Each entry in an IP address filter has an allow/refuse flag and a
specification of which addresses are covered by this entry. For the
purposes of this Setup dialogue, an IP address is always
expressed in "dotted quad" notation: a four-byte value
where each byte has its value
written out in decimal. (This is a standard convention for writing IP addresses.)
The most significant byte comes first.

There are several different ways of specifying a list entry.

:p.:hp2.Single:ehp2.

:p.This is for the case where you want to specify just one IP address.
The address must be in the dotted quad notation mentioned above.

:p.:hp2.Masked:ehp2.

:p.This is one way of specifying a group of addresses. Here, you specify a
base IP address and a binary mask, both in dotted quad notation.
A client address matches an entry if
:p.       (client IP address) :hp2.AND:ehp2. mask = (IP address in the list)
:p.where :hp2.AND:ehp2. means the bit-by-bit Boolean "logical AND" operation.
:p.Note, in particular, the two extreme cases:
:ul.
:li.If the mask is 255.255.255.255, then we are specifying an exact match between
the client IP address and the address in the list. That is, this case is
equivalent to specifying the "Single" option.
:li.If the mask were 0.0.0.0, then any IP address would match this entry. The last
entry in the list is implicitly of this form, to specify an "everything else" condition.
:eul.

:p.:hp2.CIDR (Classless Inter-Domain Routing notation):ehp2.

:p.With only rare exceptions, the "masked" notation usually uses a
mask which is a sequence of 1 bits followed by a sequence of 0 bits.
In this case it is easier to specify the number of leading 1 bits rather
than giving the mask explicitly. This what the CIDR notation does. (The
"classless" in the name refers to a relaxation of some older rules that
only allowed a few different sizes for a subnetwork.) For example, the
notation
:p.       192.168.1.0 / 24
:p.specifies a group of addresses ranging from 192.168.1.0 to 192.168.1.255.
The "24" in this example says that the group is specified by the leading
24 bits in the 32-bit address, with every possible value for the remaining 8 bits.

:p.:hp2.Range:ehp2.

:p.With this option you can specify a range of addresses. Enter the first
and last address, inclusive, in dotted quad notation.

.***********************************
.*   MODIFYING USER PERMISSIONS (PM)
.***********************************

:h2 id=PMconfiguser2.Adding and removing users
:hp2.Adding and removing users:ehp2.
:p.
The "Users" page controls who is allowed to log in to the server.
:p.
:hp2.Deleting a user:ehp2.
:p.
Select the entry you want to delete, then click on the "Delete" button.
:p.
:hp2.Adding a new user:ehp2.
:p.
Click on the "Add" button, and then proceed as for :link reftype=hd refid=PMedituser.Editing a user's permissions:elink..
:p.
:hp2.Cloning an existing user:ehp2.
:p.
First select the user whose details you want to duplicate, then click on the "Clone"
button, and then proceed as for :link reftype=hd refid=PMedituser.Editing a user's permissions:elink..
This is the same as adding a new user, except that the new user's attributes are copied from
those for an existing user.
:p.
:hp2.Editing the permissions of an existing user:ehp2.
:p.
Select the user name, click on the "Edit" button, and then follow the instructions
in the section :link reftype=hd refid=PMedituser.Editing a user's permissions:elink..
Alternatively, just double-click on the user name.
:p.
:hp2.Sorting the list:ehp2.
:p.
This is explained on the
:link reftype=hd refid=SortUsers.next page:elink.

.***********************************
.*   THE SORT USERS DIALOGUE
.***********************************

:h3 id=SortUsers.Sorting the list of users
:hp2.Sorting the list of users:ehp2.
:p.
In the "Users" notebook page, there is a button labelled "Sort". If you
press on that button, you will get a new window that again shows the
list of users, but in a different format. It does not show all the settings
for a user - for that, you need the "Edit" button on the "Users" page -
but it gives three details for each user&colon. the username, the user
category, and a timestamp for when that user last logged in. One use for
this information is to allow you to do a quick check for accounts that
should be closed because they are no longer being used.

:p.Above the list are three pushbuttons, which also serve as column headings.
Clicking on a button causes the list to be sorted using that column as a key.
A second click on the button will sort the list in descending order. If you
click on a button several times, the sort will alternate between ascending
and descending order.

:p.To ensure that the order is retained between executions of Setup,
even if you are using multiple INI files, each user is assigned a sequence
number as Setup is closing. This can slow down the close operation. The delay
is not noticeable in most cases, but it can rise to several seconds if you
have multiple INI files and many users.

.***********************************
.*   EDITING A USER'S PERMISSIONS (PM)
.***********************************

:h2 id=PMedituser.Editing a user's permissions
:hp2.Editing a user's permissions:ehp2.
:p.
You get to this point by running the Setup program and choosing any of the
options (except "Delete") on the "Users" notebook page.
:p.
The first item in the resulting dialogue is a client category - NoPassword,
Guest, User, Manager, or Template. The categories are explained in the
:link reftype=hd refid=UPconcepts.General concepts:elink. section.

:p.Underneath, there is an indication of when this user last logged in, and
to the left of that is a checkbox labelled "Allow three-cornered transfers".
(FTP only, and it will not be displayed if the user category is NoPassword, Guest,
or Template.)

:p.Next, there is a button labelled "Add IP address controls". If you
click on this button, you will get a dialogue that is similar to that on the
:link reftype=hd refid=PMsecurity.Security:elink.
page. You can then specify rules that say which IP addresses this user
is allowed to connect from. The difference between this feature and the
one on the Security page is that the rules on the Security page are
global - that is, they apply to all connections - while the present
rules supplement the global rules by allowing you to add restrictions on
a per-user basis.

:p.The next button allows you to create or modify the HideList for this
account. The HideList is explained later in this page.

:p.To the right of this are two checkboxes where you can allow this user
to use FTP, or SFTP, or both. If neither box is checked, you get an
inactive account&colon. the details
for this user are retained in the INI file, but the user
will not be able to log in. This is for the case where you want
to temporarily disable an account. It is equivalent to deleting
the user account, except that the details are saved for when you
want to re-enable the account.

:p.The next checkbox is labelled 'Limit logins'. You
should normally leave this checkbox untouched. It is for the very
special case where you want to create an account that will
self-destruct after being used a specified number of times.
See below for more details.

:p.Next you have several entry fields.
:dl break=all.
:dt.    Speed limit (FTP only)
:dd.An approximate upper bound on the average file transfer speed for this user,
in case you want to restrict how much of the processor power this user
can get. There are separate limits for uploads and downloads.
If you don't want such a control, just make the limits large
numbers. :note.There is no way to have a precise control over the
instantaneous speed. The user might well see peak speeds that are
higher than the limit; we can control only the average, not the peak.
:dt.    File size limit (FTP only)
:dd.The maximum size of a file uploaded by this user. If you enter a value
of 0, this will be treated as meaning that there is no limit.
:dt.    User limit
:dd.The maximum number of simultaneous sessions with this user name.
(If you don't want such a control, just make this number larger than the
global user limit.)
:dt.    Real name
:dd.This field is purely for your own records.  It is not used by the server.
:dt.    Username
:dd.The name that the user will use when logging in. This is limited to
256 characters; more typically it is much shorter than that. A username
containing an '@' character is legal, but should be used only in the
multidomain case (FTPServer only) where
you want to use the same name in more than one domain.
:dt.    Password
:dd.This user's password. Note that this entry field is disabled if the
user category is "NoPassword" or "Guest" or "Template".
:dt.    Notes
:dd.Use this for any purpose you wish.
:edl.

:p.The password might or might not be visible, depending on whether
you chose the 'Hide passwords' option on the Options page. If you did
enable that option, the password is shown as '*' characters. Whether
or not it is hidden, you also have an 'Encrypt password' checkbox
underneath the password. This will be explained later on this page.

:p.Warning: If you change the user name, the permissions for the previous
user name will be deleted. You should also avoid using a user name
that is the same as for some other user.

:p.There is also a checkbox labelled 'Suppress log' (FTP only). If this option
is enabled then commands from this user will be omitted from the
transaction log (except for the log entries showing the start and
end of the session). You can use this option when you want to
analyse the log for unusual behaviour and don't want to be bothered
logging the actions of trusted users. (You will possibly also want
to suppress logging of some Monitor.exe operations, but that is a special
case that is covered by the "Log SITE MNGR commands" option on the
:link reftype=hd refid=PMlogging.Logging:elink.
page.)

:p.The small window near the bottom of this dialogue gives a summary - but not a complete
description - of the top level of this user's directory tree. To see
the complete details, and to modify those details, click on the "Edit directories"
button or double-click on the summary window.

:p.Instructions for modifying the user's tree are on the
:link reftype=hd refid=PMedittree.next page:elink. of this document.

:p.Finally, there is a checkbox labelled "Use template". If you want the
directory permissions to be controlled by a template you should enable
this box, and fill in the name of the template in the entry field to
the right of the checkbox. If you do not use a template, or if you are
creating a template, you will need to select the "Edit directories"
button to define the directory tree. If you do use a template, the
tree is already defined for you, therefore the "Edit directories" option
is disabled.

:p.Note that none of your changes will be stored until you have confirmed
them with the "OK" button. To leave this dialogue without making any
changes, use the "Cancel" button.

:p.:hp2.Encrypting the password:ehp2.

:p.If you choose the 'Encrypt password' option, just under the password
entry field, the password you enter will be stored in the INI file in encrypted form.
This is an option that is selected independently for each user; that is,
you are allowed to encrypt passwords for some users but to leave them
unencrypted for other users.

:p.The encryption is a one-way encryption, and cannot be reversed. Once
you have encrypted a password, there is no way to discover what the
password was. If users lose their password, and it was encrypted, the
only solution is to give them a new password.

:p.Note that there is also a 'Hide passwords' option on the Options
page of the Setup notebook. This is a global option that applies to
all users, and you can turn it on and off. Hiding and encrypting are
two independent options, and you can choose either or both of
these options. Hiding simply controls what is visible in the Setup
notebook. Encryption is a stronger form of privacy, because the
encryption is irreversible. If you encrypt a password but do not
hide it, you will see the unencrypted password as you type it in,
but once you select the 'OK' button you will never see the original
password again.

:p.This option ONLY protects the password against someone who looks at
the password in Setup, or in the INI file. The encryption happens only
after the server has received the password. The question of what happens if
someone intercepts the communications between client and server is a
different matter. In that case,
:ul.
:li.When using FTP, the password is transmitted as plain text, unencrypted,
regardless of the "Encrypt password" option.
:li.When using SFTP, the password is sent through an encrypted channel,
again regardless of the "Encrypt password" option. It is therefore safe
against an attacker who manages to intercept the communication channel.
:eul.

:p.:hp2.The HideList:ehp2.

:p.If you press the button labelled "Add HideList" or "Modify HideList"
you will be able to specify a list of file or directory names that will always be
invisible to this user. The user is also prevented from reading,
writing, deleting, or renaming these files.
Of course you can also do this with the permission
flags. What makes the HideList different is that names in the HideList are
invisible and inaccessible in every directory that the user can see. That is, a single
entry in the HideList covers all files and subdirectories with that name in all directories.

:p.Entries in the HideList may not contain the characters '/' or '\'.
(If you try to specify those characters, they will be ignored.) That is,
you cannot specify paths in the list; only file names relative to
their own directory.

:p.The wildcard characters '?' and '*' are allowed. A '?' matches any
single character, and a '*' matches any string of zero or more
characters. So, for example, "*.htm*" would prevent the listing of
any HTML files, and "???*" would hide any names of three or more
characters in length.

:p.(Obviously you should not include the quotation marks if you
copy these examples.)

:p.As a special case, it is not possible to hide the name ".." using
the HideList. (You can, of course, hide it in particular directories
by using the V permission flag.) The reason for this rule is that
many people like to hide all file names starting with a '.', but
still want a ".." to be handled normally. An entry of ".*" in
the HideList will give this result.

:p.It is not possible to do a "change directory"
to a directory whose name is hidden by the HideList. You can,
however, change to a subdirectory of that hidden directory provided
that you know how to specify the path. (Which requires knowing what
the hidden name is.) This is consistent with the way the V directory
permission works. Of course in such a case you cannot move back up the
tree with a "CWD ..", because that would take you to the hidden
directory if it were permitted. You must instead do a "CWD ../..",
or the equivalent in your ftp client software, to jump around the
hidden directory.

:p.:hp2.Limited-use accounts:ehp2.
:p.These were called single-use accounts in an earlier version of
FtpServer, but you can now allow for more than one use. A limited-use
account is one that will be deleted or deactivated after a specified number of uses.
If you check the "Limit logins" checkbox, you will then be given
a choice of "Delete account after N logins" and "Make inactive after N logins", where you can
specify the value of N. Each time that this user logs in, the value
of N is decreased by one. Once N gets to zero, the account is deleted
or made inactive, as specified.
Before this happens, you may of course go back into Setup and change
the value of N.

:p.If you set N to 0 on this page, the account is not deleted or deactivated. (That
would be too drastic an action for something that might have been a
typing error on your part.) Instead, the account reverts back to a
normal account with an unlimited lifetime. Similarly, if the N field
is still blank, or starts with a non-numeric character, at the time
you hit the 'OK' button, this too is
interpreted as meaning an unlimited lifetime.

:p.If you want to automate the creation of limited-use accounts, see
the scripts OneTime.CMD and LimitedUse.CMD in the Tools directory for examples of how
to do it.

:p.:hp2.Converting from older formats:ehp2.
:p.Very rarely, a new version of Setup will introduce a change
in the INI file format, because for example a new option has been
introduced. When this happens, the user data have to be updated.
:p.
The Setup program will automatically perform the conversion each time
you edit a user. If you have a small number of users defined, then the
way to do the conversion is to run Setup and edit each user (without
necessarily making any changes).
:p.
If you have a large number of users, it is easier to use the LoadPRM
program to do the conversion. This can be done with the following
sequence of commands.
:xmp.
        storeprm *
        loadprm *
        del *.prm
:exmp.
:p.The final deletion can be omitted if you prefer to keep a copy of
the PRM files. If ever your INI file is damaged or destroyed, you can
use the LoadPRM utility to re-load user data from PRM files.

.***********************************
.*   EDITING A DIRECTORY TREE (PM)
.***********************************

:h2 id=PMedittree.Editing a directory tree
:hp2.Editing a directory tree:ehp2.

:p.You get to this point while
:link reftype=hd refid=PMedituser.editing a user's permissions:elink..
The picture that is shown on the screen is a representation of the
directory tree for this user. Initially it will show enough subtrees
to reveal all symbolic links, and all entries for which the user's
permissions are different from the parent node's permissions. (For
a new user, there will be nothing except an empty root node.) In
the course of editing this tree you can expand or collapse nodes to
control how much detail is shown on the screen.
:p.
The top of the screen shows both a physical path and a virtual path
for the current entry. The physical path is the true location of
the file or directory on your disk. The virtual path is the path
as the client sees it.
:p.
Each entry describes one directory or file. At the left of each line,
you will see a code consisting of one or more of the letters "VRWDN".
The meanings of these user permission codes are explained in the
:link reftype=hd refid=UPconcepts.General concepts:elink.
section.
:p.
To the right of the VRWDN code, some entries have one or more of the
following codes.
:dl.
:dt.   +
:dd.This directory is collapsed, i.e. its subdirectories (if any)
are not at present displayed on the screen.
:dt.   *
:dd.This entry is a symbolic link or pseudo-directory.
:dt.   #
:dd.This entry describes a file rather than a directory.
:dt.   ?
:dd.There is no file or directory on the disk that matches this entry.
This might mean that you have made an error in the name; alternatively,
it might mean that you are specifying a directory or file that you
haven't yet created.
:edl.
:p.
To edit the tree, you have the following options.
:ul.
:li.You can navigate through the list of directories with mouse clicks, with the
cursor up/down keys, and also with the Home, End, PageUp, and
PageDown keys.
:li.To change a permission, type one of the characters V, R, W, D, or N, or
click on the buttons with these labels.
This toggles the state of the corresponding permission code for the
currently selected directory.
:li.The "-" key or button collapses a directory by removing its subdirectories
from the screen listing. (The subdirectories are still there, and are still affected
by things like the "Propagate" option.  They simply aren't shown on the screen.)
To get the subdirectories back, type the "+" key or button.
:li.The "Add child" button adds a new child node under the current node.
:li.The "Edit" button allows you to edit the details for the current entry.
(Instructions for doing this are given later in this page.) You can also
edit by double-clicking on the entry.
This option is disabled if the current entry describes a file or directory
that is physically present on the disk.
:li.The "Delete" button deletes the current node and all of its subtrees.
This option is disabled if the current entry describes a directory
that is physically present on the disk.
:li.The "Inherit" button gives the selected entry a copy of the
current permissions of its parent.
:li.The "Propagate" button copies the permissions of the currently selected directory
to all of its subdirectories. Use this if you want to change an entire
subtree in one operation.
:eul.
:p.
When you have finished editing the permissions, click on the "Done" button
to go back to the previous dialogue. If you have made a mistake, you still have
the option of using the "Cancel" button on that previous dialogue.

:p.:hp2.Modifying the details for one entry:ehp2.

:p.The "Edit" and "Add child" options will bring up a screen window that
describes one tree node. The radio buttons at the top let you specify
one of three kinds of node.
:dl break=all.
:dt.    subdirectory or file
:dd.This refers to a subdirectory or file that is contained within the
parent directory. The "Name" field gives the name of the subdirectory or file.
Note that you cannot choose this option for the top-level directory,
because the top-level directory has no parent.
:dt.    link
:dd.This creates a symbolic link. In the "Name" field, put a name of
your own choice; this will be the name of the directory as seen by the
client. In the "Path" field, put the full physical path (including
drive letter) for this directory or file.
:dt.    pseudo-directory
:dd.This is a special case of a link, where there is no physical path.
A pseudo-directory should contain only links and other pseudo-directories.
The most common use for a pseudo-directory is as the top-level
directory for a user, containing links to the drives and/or directories
that that user will be allowed to see.
:edl.

:p.:hp2.Setting permissions for individual files:ehp2.

:p.The access permissions used by FtpServer and SFtpServer are normally given to directories,
and the permissions for a directory apply to all non-directory files in that
directory. However, Setup will let you define an entry for a non-directory
file, and give it access permissions. (It would be tedious to do this for
every file, but this feature can be used for special cases.) This gives you
a method for making the permissions for a file different from the permissions
of the directory that it is in.

:p.If you create a link or pseudo-directory with the same name as a real
file or subdirectory, your link will take precedence over the real name.
In effect, the real file or subdirectory will become invisible as far as
the user is concerned.

.***************************************
.*   MISCELLANEOUS UTILITIES
.***************************************

:h1 id=utilities.Miscellaneous utilities
:hp2.Miscellaneous utilities:ehp2.

:p.The utilities mentioned in this section are not currently included in the
SFTPServer distribution, but they are included in FTPServer. You should
find that they work equally well with SFTPServer.

.***************************************
.*   THE LOADPRM AND STOREPRM UTILITIES
.***************************************

:h2 id=loadstore.The LoadPRM and StorePRM utilities
:hp2.The LoadPRM utility:ehp2.
:p.
This utility is needed if you want to manually edit user permission files.
It copies information from a PRM file into the server's FTPD.INI. (Or into
the FTPDnnnn.INI files, if you have the "multiple INI files" option enabled.)
For example, the command
:xmp.       loadprm example
:exmp.
:p.takes the information in the file EXAMPLE.PRM and creates or updates an
entry in the INI file for a user called "example".

:p.Multiple wildcards are permitted: a '?' matches any single character, and
a '*' matches zero or more characters. (But only in the username part of the
parameter, not in the directory specification if present.) To load the information from :hp3.all:ehp3.
the PRM files in the current directory, use the command
:xmp.       loadprm *
:exmp.

:p.Optional -i or -t parameters specify the use of FTPD.INI (-i) or
FTPD.TNI (-t). If present, this parameter must come first.  For example,
:xmp.       loadprm -t abc\def\xy?u*v*
:exmp.
loads data from whatever PRM files in directory abc\def match the
pattern xy?u*v*, and puts the information into FTPD.TNI, or into
files with names FTPDnnnn.TNI if the "multiple INI files" option
is enabled in FTPD.TNI.

:p.If neither -i nor -t is specified (the usual case), the decision as
to whether to use INI or TNI format is based on the last time Setup was
run with an explicit -i or -t.

:p.You do not have to restart the server. The updated user information will
take effect the next time a user logs in.

:p.:hp2.The StorePRM utility:ehp2.
:p.
This utility creates a PRM file by copying the user information from
FTPD.INI or FTPD.TNI, as appropriate. You would use it if the INI file already contains user data
that you want to edit manually. For example, the command
:xmp.       storeprm example
:exmp.
:p.takes the information in the INI file for the user called "example",
and uses it to create a file EXAMPLE.PRM. (If EXAMPLE.PRM already exists,
the original copy is renamed EXAMPLE.BAK.)

:p.StorePRM has the same parameter options as LoadPRM.

.***************************************************
.*   THE REMOVE AND ONETIME AND LIMITEDUSE UTILITIES
.***************************************************

:h2.The REMOVE, ONETIME, and LIMITEDUSE utilities
:hp2.The REMOVE utility:ehp2.

:p.The file REMOVE.CMD is a Rexx script for deleting a user
from the database. Normally you would use the Setup utility
to remove a user, so most people will not need REMOVE.CMD.
It is, however, useful if your application requires you to
add and remove users dynamically. You can use LoadPRM to
add a user, and REMOVE to remove a user.

:p.To use REMOVE.CMD, you simply execute the command
:xmp.       REMOVE username

:exmp.
where 'username' is the name of the user you want to remove.

:p.:hp2.The ONETIME and LIMITEDUSE utilities:ehp2.

:p.The file ONETIME.CMD is a Rexx script for creating a "use once"
account in the user database. You can also create such an
account using the Setup utility, but the Rexx script is useful
for situations where you want an automated system. For example,
this script can be called from your web server software if you
have a need to do downloads that are controlled by options on
a web page.

:p.A "use once" account is similar to any other user account,
except that the server automatically deletes the account the
first time it is used.

:p.To use ONETIME.CMD, you simply execute the command
:xmp.       ONETIME username password

:exmp.
where the two parameters give the username and password for the
account you are creating.

:p.The script ONETIME.CMD includes a hard-coded permission string
that controls which directories will be accessible to this user.
You should edit this when installing the script. Of course, you
are also free to make any other changes to the script to handle
any special need that your application might have.

:p.LIMITEDUSE.CMD is a variant that allows several logins before
the account is removed.

.***************************************
.*   THE LOGANALYSIS UTILITY
.***************************************

:h2.The LogAnalysis utility
:hp2.The LogAnalysis utility:ehp2.

:p.LogAnalysis.exe is a utility to produce a summary from the
FtpServer user log.  You run it with the command

:xmp.        LogAnalysis logfilename:exmp.

:p.where "logfilename" is the name of the user log file. If no
parameter is supplied, it assumes that the log file name is
FTPUSERS.LOG.

:p.The results are written to standard output. You can redirect
this to a file, for example

:xmp.        LogAnalysis AUGUST.LOG >August.summary
:exmp.

:p.Note that very large log files can cause this program to fail
by running out of memory. If this happens, use a text editor to
split your log file into two or more smaller files.

.***********************************

:euserdoc.

