:userdoc.
:title.SSH Server documentation
:docprof toc=123.

.***********************************
.*   INTRODUCTION
.***********************************

:h1 res=1000 id=1000 global.Introduction

:p.The base layer of this software is my own implementation of the secure shell called SSH. It is based
on RFC 4253 for the transport layer, RFC4252 for the authentication layer, and
RFC4254 for the connection protocol. This document is more for my own information
than for other people, to make it easy to keep track of what I'm up to.

:p.I am not particularly interested in SSH as such, but in developing an implementation
of SFTP, the secure file transfer protocol. Since SFTP, as normally implemented, is an
application that sits on top of the SSH support, we need SSH as a starting point. In
principle I could start with an existing SSH implementation like OpenSSH, but I have not
had much luck in that direction. Although I've written a lot of encryption and decryption
code, I'm still no good at decrypting poorly documented C code. Anyway, I already had a
lot of the basic modules that I'd developed for other projects, so putting them
together into an SSH package was a feasible project.

:p.The main executables are sftpd.exe, the server, and sftp.exe, a client.
These share a great deal of common code; at the transport layer level, I have not so far
felt any need to separate the "client" and "server" parts of the modules. The client
began life as a test program to exercise the server, but has since evolved
into a proper sftp client.

:p.As you will see from the :link reftype=hd refid=projectstatus.status:elink.
page, the SSH component of the overall package is essentially complete.

:p.The author of this package is Peter Moylan, peter@pmoylan.org.

:p.As always, there are no guarantees that this will work for anyone else.

.***********************************
.*   PROJECT STATUS
.***********************************

:h1 id=projectstatus.Overall project status
:hp2.Overall project status:ehp2.

:p.The software is organised into a number of layers.

:p.The transport layer, which does all of the encryption and decryption, and a number
of related jobs, is complete and working.

:p.The authentication layer does user authentication. The present version is small
and simple, but is sufficiently complete for our present purposes. This too is working.

:p.The connection layer is now complete.

:p.The application layer - the SFTP server itself - is documented in a
:link reftype=hd database='SFTPServer.INF' refid=1000.separate manual:elink..

.***********************************
.*   THE KEYGEN UTILITY
.***********************************

:h1 id=keygenutility.The keygen utility
:hp2.The keygen utility:ehp2.

:p.The server needs a public/private RSA key pair so that it can produce a
cryptographic signature during the initial negotiation with a client. The program
keygen.exe creates this key pair, so you need to run it before your first run
of the server. It is invoked with
:xmp.
       keygen -b nnnn
:exmp.
where nnnn is the desired number of bits in the public key. If it is not specified
then the default value is 1536. The reason for this default is that many people
now think that 1024 bits or fewer is not enough for security. You are of course free
to experiment. The result is stored in a subdirectory with name .ssh.

:p.The starting point for generating the key pair is finding two large random prime
numbers. This is a very slow operation because most of the candidates turn out not to
be prime, and discovering that is time-consuming. (The tests I've done suggest that
the execution time for keygen is an exponential function of the number of bits.)
Luckily, this only needs to be done once.
:p.

.***********************************
.*   THE TRANSPORT LAYER
.***********************************

:h1 id=transportlayer.The Transport Layer
:hp2.The transport layer:ehp2.

:p.The transport layer is the most technically complex part of the software.
Its function is to move encrypted packets back and forth between server and
client, and a fairly complicated negotiation of what algorithms to use.

:p.The following pages show the implementation state of the various
important classes of algorithms.

:ul.
:li.:link reftype=hd refid=compression.Compression:elink.
:li.:link reftype=hd refid=encryption.Encryption:elink.
:li.:link reftype=hd refid=MAC.Message authentication:elink.
:li.:link reftype=hd refid=kex.Key exchange:elink.
:li.:link reftype=hd refid=pka.Public key algorithms:elink.
:li.:link reftype=hd refid=hash.Hash algorithms:elink.
:eul.

.***********************************
.*   COMPRESSION
.***********************************

:h2 id=compression.Compression algorithms
:hp2.Compression algorithms:ehp2.

:p.The only compression type implemented so far is "none" - that is, no
compression. Hooks are in place to allow one or more real compression
algorithms to be used, but at present this has a low priority. This,
I believe, is the only aspect of SSH algorithm negotiation where "none"
is an acceptable choice, and indeed the usual choice.

.***********************************
.*   ENCRYPTION
.***********************************

:h2 id=encryption.Encryption algorithms
:hp2.Encryption algorithms:ehp2.

:p.Encryption is the basic encipherment applied to every packet, except
during the initial negotiation of what algorithms to use. Here we
support several versions of AES (advanced encryption standard):

:ul.
:li.aes128-ctr
:li.aes192-ctr
:li.aes256-ctr
:li.aes128-cbc
:li.aes192-cbc
:li.aes256-cbc
:eul.

:p.and one version of DES (data encryption standard):

:ul.
:li.3des-cbc (also known as 3-DES or triple-DES)
:eul.

:p.All of these have been tested and are working well. It might turn
out to be necessary to add to this collection, because a number of
other encryption algorithms seem to be popular.

:p.:hp2.Remark:ehp2.. It turns out that the aesxxx-ctr algorithms used by SSH
are different from the aesxx-ctr algorithms described by earlier standards.
I have now added the new mode, calling it SDCTR mode to distinguish it from
the previously defined CTR mode, and my implementation passes the rather
skimpy set of published acceptance tests.

:p.Documentation on the algorithms used with SSH are, in general, of much
poorer quality than the earlier NIST encryption standards. I have found myself
doing a lot of trial and error to resolve the ambiguities in the newer standards.

.***********************************
.*   MESSAGE AUTHENTICATION
.***********************************

:h2 id=MAC.Message authentication
:hp2.Message authentication:ehp2.

:p.MAC (message authentication code) bytes are appended to every packet.
The MAC is a function of a shared key, the packet sequence number, and
the packet data before encryption. This is to allow detection of packets
that have been altered during transmission. Currently the following
MAC algorithms are supported.

:ul.
:li.hmac-sha1
:li.hmac-sha1-96
:li.hmac-sha2-256
:li.hmac-sha2-256-96
:li.hmac-sha2-384
:li.hmac-sha2-384-96
:li.hmac-sha2-512
:li.hmac-sha2-512-96
:li.hmac-md5
:li.hmac-md5-96
:eul.

:p.These are all based on code that has been extensively tested.

.***********************************
.*   KEY EXCHANGE
.***********************************

:h2 id=kex.Key exchange
:hp2.Key exchange:ehp2.

:p.At the beginning of an SSH session (and also at other times, if either
partner wants to re-negotiate) the client and server negotiate which
algorithms they are going to use. The first part of this is elementary:
in each algorithm category, one partner offers a list of algorithms it
is willing to use, and the other partner chooses from the list. The
next part is more complicated: the server and client exchange information
that will lead them to separately calculate keys that will be used for
other algorithms.

:p.At present all of our key exchange algorithms are based
on the Diffie-Helman method:
:ul.
:li.diffie-hellman-group1-sha1
:li.diffie-hellman-group14-sha1
:li.diffie-hellman-group14-sha256
:li.diffie-hellman-group15-sha512
:eul.

:p.I am confident that these are correctly deriving the temporary shared key K
and the shared hash H, because if either of these were wrong the server's
signature would be rejected.

:p.The final step of key exchange is to generate six keys, three for each
direction, that will be used for other algorithms:

:ul.
:li.Initialisation vector for the encryption of packets.
:li.Encryption key, also for use by the encryption algorithm.
:li.Integrity key for the use of the MAC algorithm.
:eul.

:p.Since encrypted packets are now being successfully decrypted, we can be
confident that the key generation is working.

.***********************************
.*   PUBLIC KEY ALGORITHMS
.***********************************

:h2 id=pka.Public key algorithms
:hp2.Public key algorithms:ehp2.

:p.Public key encryption, where a message encrypted with the public key can
be decrypted by the private key, and vice versa, is secure but computationally very
expensive. SSH does not use this for encrypting packets, but for just two
purposes:
:ul.
:li.As a certification that the server is who it claims to be. For this purpose, an
SSH client keeps a collection of public keys for the servers it wants to contact.
:li.To encrypt a signature that the server generates during key exchange. That
signature lets the client know that both partners have generated the same exchange
hash, which is a vital step in generating the keys that several algorithms need.
:eul.

:p.At present we have support for only three public key algorithms:

:ul.
:li.ssh-rsa
:li.rsa-sha2-256
:li.rsa-sha2-512
:eul.

:p.These all use the RSA form of public key, differing only in the way
signatures are generated. At some stage I should probably add one or
more elliptic curve signature algorithms, but at present this does not
have a high priority.

.***********************************
.*   HASH ALGORITHMS
.***********************************

:h2 id=hash.Hash functions
:hp2.Hash functions:ehp2.

:p.A hash function creates a "digest" of an arbitrary block of data, with the
digest being typically 12 to 64 bytes in length, depending on which hash function
it is. This is a one-way encryption, that serves as a plausibility check that two data blocks
are identical.

:p.Hash functions are not directly negotiated during key exchange, but turn up as
part of the specification of other algorithms. For example, Diffie-Hellman key
exchange requires a sha-1 hash to be calculated a number of times, and the
specification of the rsa-sha2-512 public key algorithm requires the sha2-512 hash
function to be used as one step in creating a signature.

:p.We support the following hash functions:

:ul.
:li.md5
:li.sha-1
:li.sha-2-256
:li.sha-2-512
:eul.

:p.These all use well-tested code. Other hash functions could easily be added to this collection,
but they are not necessary unless and until some newly adopted algorithm needs them.


.***********************************
.*   THE AUTHENTICATION LAYER
.***********************************

:h1 id=authenticationlayer.The authentication layer
:hp2.The authentication layer:ehp2.

:p.RFC 4252 specifies three kinds of authentication: public key, password, and host based.
Of these, only public key is required, and I suppose I should implement it eventually.
I believe, though, that login with a password is most appropriate for an FTP application.
This, therefore, is the only method I have implemented so far.

:p.Note that the packet containing username and password is passed from client to
server in encrypted form. This is not at all like sending a plain-text password.
Even if the traffic is intercepted, it would be very hard for an attacker to
discover the username and password.

:p.We also allow for a class of users that are not required to supply a password.

.***********************************
.*   THE CONNECTION LAYER
.***********************************

:h1 id=connectionlayer.The connection layer
:hp2.The connection layer:ehp2.

:p.In this layer, as coded in the module called Channels, we support the basic operations
of opening and closing a channel, and sending and receiving data through a channel. Since
the goal is to support the SFTP protocol, and only that, many of the channel requests
documented in RFC 4254 are ignored. These are for things like terminal emulation, running a remote shell,
port forwarding, and similar features. The only feature of that kind that is implemented is
handling the request to start an SFTP session.

:p.In the event that I ever need to implement those other features, I will probably write
a separate version of this layer for each application, rather than building a single
do-everything program.

.***********************************
.*   NOTES ON WORK IN PROGRESS
.***********************************

:h1 id=totry.Notes on work in progress

:p.At present we have no SSH sub-projects in progress.

:euserdoc.

