MODULE keygentime;

        (********************************************************)
        (*                                                      *)
        (*           Working out keygen execution time          *)
        (*                                                      *)
        (*           The results are in file KGTIME.CSV         *)
        (*                                                      *)
        (*  Programmer:         P. Moylan                       *)
        (*  Last edited:        3 July 2021                     *)
        (*  Status:             OK                              *)
        (*                                                      *)
        (********************************************************)


IMPORT BigNum;

FROM TimeConv IMPORT
    (* proc *)  time;

FROM FileOps IMPORT
    (* type *)  ChanId,
    (* proc *)  Exists, DeleteFile, MoveFile, OpenNewFile, CloseFile,
                FWriteChar, FWriteString, FWriteLn;

FROM RSAKeys IMPORT
    (* type *)  RSAKeyType,
    (* proc *)  DiscardKey;

FROM RSAGenerate IMPORT
    (* proc *)  RSA_Generate;

FROM LowLevel IMPORT
    (* proc *)  EVAL;

(************************************************************************)
(*                            KEY GENERATION                            *)
(************************************************************************)

PROCEDURE FWriteCard (cid: ChanId;  N: CARDINAL);

    BEGIN
        IF N > 9 THEN
            FWriteCard (cid, N DIV 10);
            N := N MOD 10;
        END (*IF*);
        FWriteChar (cid, CHR(ORD('0') + N));
    END FWriteCard;

(************************************************************************)

PROCEDURE RunTheTests;

    CONST
        maxwords = 64;
        numtests = 10;
        filename = "KGTIME.CSV";  bakname = "KGTIME.CSV.bak";

    VAR j, k, bits, t0, t1: CARDINAL;
        cid: ChanId;
        key: RSAKeyType;

    BEGIN
        IF Exists(filename) THEN
            IF Exists(bakname) THEN
                DeleteFile (bakname);
            END (*IF*);
            EVAL (MoveFile (filename, bakname));
        END (*IF*);
        cid := OpenNewFile (filename, FALSE);
        FWriteString (cid, "bits,seconds");  FWriteLn (cid);
        FOR j := 2 TO maxwords DO
            bits := 32*j;
            t0 := time();
            FOR k := 1 TO numtests DO
                key := RSA_Generate(bits);
                DiscardKey (key);
            END (*FOR*);
            t1 := time();
            IF t1 < t0 THEN t0 := t1 END(*IF*);
            FWriteCard (cid, bits);  FWriteChar (cid, ',');
            FWriteCard (cid, (t1 - t0 + numtests DIV 2) DIV numtests);
            FWriteLn (cid);
        END (*FOR*);
        CloseFile (cid);
    END RunTheTests;

(************************************************************************)

BEGIN
    RunTheTests;
END keygentime.

